@@ -1,5 +1,5 @@
 /* source: xio-ip.c */
-/* Copyright Gerhard Rieger */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for IP related functions */
@@ -149,6 +149,9 @@ int xiogetaddrinfo(const char *node, const char *service,
    memset(sau, 0, *socklen);
    sau->soa.sa_family = family;
 
+   if (service && service[0]=='\0') {
+      Error("empty port/service");
+   }
    /* if service is numeric we don't want to have a lookup (might take long
       with NIS), so we handle this specially */
    if (service && isdigit(service[0]&0xff)) {
@@ -236,7 +239,8 @@ int xiogetaddrinfo(const char *node, const char *service,
 
       if ((error_num = Getaddrinfo(node, service, &hints, &res)) != 0) {
 	 Error7("getaddrinfo(\"%s\", \"%s\", {%d,%d,%d,%d}, {}): %s",
-		node, service, hints.ai_flags, hints.ai_family,
+		node?node:"NULL", service?service:"NULL",
+		hints.ai_flags, hints.ai_family,
 		hints.ai_socktype, hints.ai_protocol,
 		(error_num == EAI_SYSTEM)?
 		strerror(errno):gai_strerror(error_num));
@@ -461,7 +465,8 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 			char *nambuff, int namlen,
 			char *envbuff, int envlen,
 			char *valbuff, int vallen) {
-   const char *cmsgtype, *cmsgname = NULL, *cmsgenvn = NULL, *cmsgfmt = NULL;
+   int cmsgctr = 0;
+   const char *cmsgtype, *cmsgname = NULL, *cmsgenvn = NULL;
    size_t msglen;
    char scratch1[16];	/* can hold an IPv4 address in ASCII */
 #if WITH_IP4 && defined(IP_PKTINFO) && HAVE_STRUCT_IN_PKTINFO
@@ -503,7 +508,7 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
       return STAT_OK;
 #endif /* defined(IP_PKTINFO) && HAVE_STRUCT_IN_PKTINFO */
 #endif /* WITH_IP4 */
-#ifdef IP_RECVERR
+#if defined(IP_RECVERR) && HAVE_STRUCT_SOCK_EXTENDED_ERR
    case IP_RECVERR: {
       struct sock_extended_err *err =
 	 (struct sock_extended_err *)CMSG_DATA(cmsg);
@@ -521,7 +526,7 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 	       err->ee_code, '\0', err->ee_info, '\0', err->ee_data);
       return STAT_OK;
    }
-#endif /* IP_RECVERR */
+#endif /* defined(IP_RECVERR) && HAVE_STRUCT_SOCK_EXTENDED_ERR */
 #ifdef IP_RECVIF
    case IP_RECVIF: {
       /* spec in FreeBSD: /usr/include/net/if_dl.h */
@@ -551,17 +556,17 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
 #ifdef IP_RECVOPTS
    case IP_RECVOPTS:
 #endif
-      cmsgtype = "IP_OPTIONS"; cmsgname = "options"; cmsgfmt = NULL; break;
+      cmsgtype = "IP_OPTIONS"; cmsgname = "options"; cmsgctr = -1; break;
    case IP_TOS:
-      cmsgtype = "IP_TOS";     cmsgname = "tos"; cmsgfmt = "%u"; break;
+      cmsgtype = "IP_TOS";     cmsgname = "tos"; cmsgctr = msglen; break;
    case IP_TTL: /* Linux */
 #ifdef IP_RECVTTL
    case IP_RECVTTL: /* FreeBSD */
 #endif
-      cmsgtype = "IP_TTL";     cmsgname = "ttl"; cmsgfmt = "%u"; break;
+      cmsgtype = "IP_TTL";     cmsgname = "ttl"; cmsgctr = msglen; break;
    }
    /* when we come here we provide a single parameter
-      with type in cmsgtype, name in cmsgname, printf format in cmsgfmt */
+      with type in cmsgtype, name in cmsgname, value length in msglen */
    *num = 1;
    if (strlen(cmsgtype) >= typlen)  rc = STAT_WARNING;
    typbuff[0] = '\0'; strncat(typbuff, cmsgtype, typlen-1);
@@ -573,10 +578,14 @@ int xiolog_ancillary_ip(struct cmsghdr *cmsg, int *num,
    } else {
       envbuff[0] = '\0';
    }
-   if (cmsgfmt != NULL) {
-      snprintf(valbuff, vallen, cmsgfmt, *(unsigned char *)CMSG_DATA(cmsg));
-   } else {
-      xiodump(CMSG_DATA(cmsg), msglen, valbuff, vallen, 0);
+   switch (cmsgctr) {
+   case sizeof(char):
+      snprintf(valbuff, vallen, "%u", *(unsigned char *)CMSG_DATA(cmsg)); break;
+   case sizeof(int):
+      snprintf(valbuff, vallen, "%u",    (*(unsigned int *)CMSG_DATA(cmsg))); break;
+   case 0:
+      xiodump(CMSG_DATA(cmsg), msglen, valbuff, vallen, 0); break;
+   default: break;
    }
    return rc;
 }
