@@ -1,5 +1,5 @@
-/* $Id: xiomodes.h,v 1.7 2002/09/29 19:37:10 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001, 2002 */
+/* source: xiomodes.h */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xiomodes_h_included
@@ -15,18 +15,24 @@
 #include "xio-creat.h"
 #include "xio-gopen.h"
 #include "xio-pipe.h"
+#if _WITH_SOCKET
 #include "xio-socket.h"
 #include "xio-listen.h"
 #include "xio-unix.h"
 #include "xio-rawip.h"
+#include "xio-interface.h"
 #include "xio-ip.h"
+#if WITH_IP4
 #include "xio-ip4.h"
+#endif /* WITH_IP4 */
 #include "xio-ip6.h"
 #include "xio-ipapp.h"
 #include "xio-tcp.h"
 #include "xio-udp.h"
+#include "xio-sctp.h"
 #include "xio-socks.h"
 #include "xio-proxy.h"
+#endif /* _WITH_SOCKET */
 #include "xio-progcall.h"
 #include "xio-exec.h"
 #include "xio-system.h"
@@ -34,5 +40,9 @@
 #include "xio-readline.h"
 #include "xio-pty.h"
 #include "xio-openssl.h"
+#include "xio-tcpwrap.h"
+#include "xio-ext2.h"
+#include "xio-tun.h"
+#include "xio-streams.h"
 
 #endif /* !defined(__xiomodes_h_included) */
